@@ -1,6 +1,4 @@
-from . import dependency
-from .exceptions import InvalidCommand
-from .cmd_base import DoitCmdBase
+from .cmd_base import DoitCmdBase, check_tasks_exist, subtasks_iter
 
 
 class Forget(DoitCmdBase):
@@ -13,27 +11,25 @@ class Forget(DoitCmdBase):
     def _execute(self):
         """remove saved data successful runs from DB
         """
-        dependency_manager = dependency.Dependency(self.dep_file)
+        dependency_manager = self.dep_class(self.dep_file)
+
         # no task specified. forget all
         if not self.sel_tasks:
             dependency_manager.remove_all()
             self.outstream.write("forgeting all tasks\n")
+
         # forget tasks from list
         else:
             tasks = dict([(t.name, t) for t in self.task_list])
+            check_tasks_exist(tasks, self.sel_tasks)
+
             for task_name in self.sel_tasks:
-                # check task exist
-                if task_name not in tasks:
-                    msg = "'%s' is not a task."
-                    raise InvalidCommand(msg % task_name)
-                # for group tasks also remove all tasks from group.
-                group = [task_name]
-                while group:
-                    to_forget = group.pop(0)
-                    if not tasks[to_forget].actions:
-                        # get task dependencies only from group-task
-                        group.extend(tasks[to_forget].task_dep)
+                # for group tasks also remove all tasks from group
+                sub_list = [t.name for t in subtasks_iter(tasks,
+                                                          tasks[task_name])]
+                for to_forget in [task_name] + sub_list:
                     # forget it - remove from dependency file
                     dependency_manager.remove(to_forget)
                     self.outstream.write("forgeting %s\n" % to_forget)
+
         dependency_manager.close()
